package pt.moth.dependencies;

import pt.moth.util.variables.IfConditionVar;
import pt.moth.util.variables.ParamRef;
import pt.moth.util.variables.ParamVar;
import pt.moth.util.variables.ReturnRef;
import pt.moth.util.variables.ReturnVar;
import pt.moth.util.variables.Variable;
import soot.SootMethod;

/**
 * This Class represents a specific version of a variable
 * @author VascoP
 *
 */
public class VarVersion {

	protected Variable var;
	// Hashcode of the last statement where this variable was defined
	protected int hashCode;
	
	public VarVersion(Variable var, int hashCode){
		this.var = var.copy();
		this.hashCode = hashCode;
	}

	// Constructors with special variables that do not need hashCode:

	
	public VarVersion(IfConditionVar var) {
		this.var = var.copy();
		this.hashCode = 0;
	}
	public VarVersion(ParamVar var){
		this.var = var.copy();
		this.hashCode = 0;
	}
	public VarVersion(ParamRef var){
		this.var = var.copy();
		this.hashCode = 0;
	}
	public VarVersion(ReturnVar var){
		this.var = var.copy();
		this.hashCode = 0;
	}
	public VarVersion(ReturnRef var){
		this.var = var.copy();
		this.hashCode = 0;
	}
	
	// Methods:
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + hashCode;
		result = prime * result + ((var == null) ? 0 : var.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		VarVersion other = (VarVersion) obj;
		if (hashCode != other.hashCode)
			return false;
		if (var == null) {
			if (other.var != null)
				return false;
		} else if (!var.equals(other.var))
			return false;
		return true;
	}

	public Variable getVariable() {
		return var;
	}

	public VarVersion copy() {
		return new VarVersion(var, hashCode);
	}
	
	public SootMethod getSootMethod(){
		return var.getSootMethod();
	}

	public int getHashCode() {
		return hashCode;
	}
	
	@Override
	public String toString(){
		return "("+var+","+hashCode+")";
	}
}
