package pt.moth.dependencies.firsPhase;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import pt.moth.dependencies.VarDepCollector;
import pt.moth.inter.ForwardInterproceduralAnalysis;
import pt.moth.util.Pair;


import soot.Body;
import soot.SootMethod;
import soot.jimple.InvokeExpr;
import soot.toolkits.graph.BriefUnitGraph;
import soot.toolkits.graph.UnitGraph;

public class InterAnalysis extends ForwardInterproceduralAnalysis<GraphLattice> {
	
	Map<Pair<String,String>,GraphLattice> method_lats = new HashMap<Pair<String,String>, GraphLattice>();
	
	public InterAnalysis() {
		super();
		analyze();
		GraphLattice mainLattice = mergeGraphs();
		VarDepCollector.storeDataDeps(mainLattice);
	}
	
	@Override
	protected GraphLattice analyzeMethod(Body activeBody, GraphLattice paramValues) {
		UnitGraph g = new BriefUnitGraph(activeBody);
		GraphLattice res = new IntraAnalysis(g, paramValues,this).getFlowAtEnd();

		// Prune all local Edges (L1 -> L2)
//		res.getGraph().pruneLocalVertices();
		SootMethod method = activeBody.getMethod();
		String methodSig = method.getSignature();
		SootMethod atomicMethod = res.getAtomicMethod();
		String atomicSig = (atomicMethod == null)?"":atomicMethod.getSignature();
		
		// Store the returned Lattice
		method_lats.put(new Pair<String, String>(methodSig,atomicSig), res);
		return new GraphLattice();
	}

	private GraphLattice mergeGraphs() {
		GraphLattice out = new GraphLattice();
		for(GraphLattice graph : method_lats.values()){
			out.union(graph);
		}
		return out;
	}
	
	public String printAllGraphLattices(){
		String r = "";
		for(Entry<Pair<String, String>, GraphLattice> e : method_lats.entrySet()){
			System.out.println("\n("+e.getKey().getFirst()+","+e.getKey().getSecond()+")");
			System.out.println(e.getValue()+"\n---------------------------------------------");
		}
		return r;
	}
	@Override
	protected void copy(GraphLattice source, GraphLattice dest) {
		source.copy(dest);
	}

	@Override
	protected GraphLattice defaultParameterValues(SootMethod method) {
		return new GraphLattice();
	}

	@Override
	protected GraphLattice defaultReturnValue(SootMethod method) {
		return new GraphLattice();
	}

	@Override
	protected GraphLattice handleSkippedMethodInvocation(SootMethod callerMethod,
			GraphLattice out, InvokeExpr invokeExpr, SootMethod invokedMethod) {
		return (GraphLattice)out.clone();
	}

	@Override
	protected void merge(GraphLattice in1, GraphLattice in2, GraphLattice out) {
		in1.union(in2, out);
	}

	@Override
	protected GraphLattice newInitialFlow() {
		return new GraphLattice();
	}

	@Override
	protected void populateParameters(SootMethod callerMethod,
			SootMethod calleeMethod, InvokeExpr invokeExpr, GraphLattice out,
			GraphLattice parameter) {
		parameter.union(out);
	}

	@Override
	protected boolean shouldAnalyzeMethod(SootMethod method) {
		if (method.getDeclaringClass().isPhantom()) {
			return false;
		}
		else if (!method.hasActiveBody()) {
			return false;
		}
		return true;
	}

}
