package pt.moth.dependencies.firsPhase;


import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import pt.moth.dependencies.VarVersion;
import pt.moth.instanceTypeAnalysis.UnitsCollector;
import pt.moth.inter.MethodUtils;
import pt.moth.util.ArrayLocal;
import pt.moth.util.ArrayRef;
import pt.moth.util.ImmutableManager;
import pt.moth.util.graphs.DepEdge;
import pt.moth.util.variables.GlobalVar;
import pt.moth.util.variables.IfConditionVar;
import pt.moth.util.variables.IfGlobalVar;
import pt.moth.util.variables.IfLocalVar;
import pt.moth.util.variables.LocalVar;
import pt.moth.util.variables.NativeGlobalVar;
import pt.moth.util.variables.ParamRef;
import pt.moth.util.variables.ParamVar;
import pt.moth.util.variables.ReturnRef;
import pt.moth.util.variables.ReturnVar;
import pt.moth.util.variables.Variable;



import soot.Local;
import soot.PrimType;
import soot.Scene;
import soot.SootClass;
import soot.SootMethod;
import soot.SootMethodRef;
import soot.Type;
import soot.Unit;
import soot.Value;
import soot.jimple.Constant;
import soot.jimple.FieldRef;
import soot.jimple.InvokeExpr;
import soot.jimple.MonitorStmt;
import soot.jimple.Stmt;
import soot.jimple.internal.AbstractBinopExpr;
import soot.jimple.internal.AbstractInstanceOfExpr;
import soot.jimple.internal.AbstractUnopExpr;
import soot.jimple.internal.ImmediateBox;
import soot.jimple.internal.JArrayRef;
import soot.jimple.internal.JAssignStmt;
import soot.jimple.internal.JCastExpr;
import soot.jimple.internal.JGotoStmt;
import soot.jimple.internal.JIdentityStmt;
import soot.jimple.internal.JIfStmt;
import soot.jimple.internal.JInterfaceInvokeExpr;
import soot.jimple.internal.JLookupSwitchStmt;
import soot.jimple.internal.JNewExpr;
import soot.jimple.internal.JNewMultiArrayExpr;
import soot.jimple.internal.JReturnStmt;
import soot.jimple.internal.JReturnVoidStmt;
import soot.jimple.internal.JSpecialInvokeExpr;
import soot.jimple.internal.JStaticInvokeExpr;
import soot.jimple.internal.JTableSwitchStmt;
import soot.jimple.internal.JThrowStmt;
import soot.jimple.internal.JVirtualInvokeExpr;
import soot.jimple.internal.JimpleLocal;
import soot.toolkits.graph.DirectedGraph;
import soot.toolkits.graph.UnitGraph;
import soot.toolkits.scalar.ForwardFlowAnalysis;

public class IntraAnalysis extends ForwardFlowAnalysis<Unit, GraphLattice> {

	protected InterAnalysis interAnalysis;
	protected SootMethod thisMethod;
	protected GraphLattice entryLattice;
	protected Map<LocalVar, ArrayRef> arrayVariables;
	protected Map<LocalVar, ArrayLocal> arrayLocals;
	// Keeps the global Objects kept by some local object
	protected Map<LocalVar, GlobalVar> sameObjects;

	public IntraAnalysis(DirectedGraph<Unit> graph, GraphLattice paramValues,
			InterAnalysis interAnalysis) {
		super(graph);
		this.interAnalysis = interAnalysis;
		this.thisMethod = ((UnitGraph) graph).getBody().getMethod();
		this.arrayVariables = new HashMap<LocalVar, ArrayRef>();
		this.arrayLocals = new HashMap<LocalVar, ArrayLocal>();
		this.sameObjects = new HashMap<LocalVar, GlobalVar>();
		entryLattice = paramValues;
		doAnalysis();
	}

	@SuppressWarnings("unchecked")
	@Override
	protected void flowThrough(GraphLattice inValue, Unit unit,
			GraphLattice outValue) {
		inValue.copy(outValue);
		SootClass superClass = thisMethod.getDeclaringClass();
		while (superClass.hasSuperclass()
				&& !superClass.getSuperclass().toString()
						.contains("java.lang.Object")) {
			superClass = superClass.getSuperclass();
		}
		if (superClass.toString().contains("Throwable")) {
			return;
		}
		// @@@@@@@@ Analyse if not exception @@@@@@@@@

		Stmt stmt = (Stmt) unit;
		if (stmt.containsInvokeExpr()) {
			InvokeExpr expr = stmt.getInvokeExpr();
			SootMethod method = expr.getMethod();
			SootClass invokeClass = method.getDeclaringClass();

			// Compute left and base Vars "left = base.method(p1,p2,...)"
			LocalVar leftVar = null;
			if (stmt instanceof JAssignStmt) {
				Value leftop = ((JAssignStmt) stmt).getLeftOp();
				if (leftop instanceof JimpleLocal)
					leftVar = new LocalVar(thisMethod, (JimpleLocal) leftop);

				if (!ImmutableManager.isImmutable(invokeClass)) {
					ReturnRef returnVar = new ReturnRef(thisMethod, stmt);
					addDepEdge(returnVar, leftVar, outValue, stmt);
				}
			}
			JimpleLocal base = getInvokeExprLocal(stmt.getInvokeExpr());
			boolean isNull = base == null || base.getName().equals("this");
			LocalVar baseVar = isNull ? null : new LocalVar(thisMethod, base);

			// Link up the left and the base vars
			if (baseVar != null && leftVar != null) {
				addDepEdge(baseVar, leftVar, outValue, stmt);
			}

			// Link up the method arguments with leftVar and baseVar
			Iterator<Value> args = expr.getArgs().iterator();
			linkupArgsAndObject(args, baseVar, leftVar, outValue, stmt);

			// Link up the parameters and the method's local variables
			args = expr.getArgs().iterator();
			linkupArgsAndLocals(stmt, method, args, outValue);

			if (!interAnalysis.shouldAnalyzeMethod(method)) {
				Set<String> implementations = UnitsCollector
						.getImplementations(stmt);
				// If has implementations, cut edges in the callGraph
				if (implementations.size() > 0) {
					Set<String> methodsToStay = new HashSet<String>();
					SootMethodRef ref;
					for (String i : implementations) {
						ref = Scene.v().makeMethodRef(new SootClass(i),
								method.getName(), method.getParameterTypes(),
								method.getReturnType(), method.isStatic());
						methodsToStay.add(ref.resolve().getSignature());
					}
					MethodUtils.changeEdges(thisMethod, stmt, methodsToStay);
				}
				// Don't have implementations, and we can not analyse this
				// method
				else {
					if (method.isNative() && inValue.hasAtomicMethod()) {
						treateNativeDependencies(method,
								inValue.getAtomicMethod(), outValue, stmt);
					}
					return;
				}
			}
			// Invoke the called method
			GraphLattice param = new GraphLattice();
			param.setGlobalHashCodes(inValue, method);
			
			SootMethod atomicMethod = inValue.getAtomicMethod();
			if (atomicMethod != null) {
				param.setAtomicMethod(atomicMethod);
			} else if (MethodUtils.methodIsAtomic(thisMethod)) {
				param.setAtomicMethod(thisMethod);
			}
			if (!ImmutableManager.isImmutable(invokeClass))
				interAnalysis.handleMethodInvocation(thisMethod, param, stmt);

		}
		// This instruction is an assignment x = 4 or z = x
		else if (unit instanceof JAssignStmt) {
			Value leftop = ((JAssignStmt) unit).getLeftOp();
			Value rightop = ((JAssignStmt) unit).getRightOp();
			/**
			 * "$r0[$i0] = m": - array($r0) <-- $i0 - array($r0) <-- m -
			 * array($r0) <-- dep($r0)
			 */
			if (leftop instanceof JArrayRef) {
				LocalVar r0 = new LocalVar(thisMethod,
						(JimpleLocal) ((JArrayRef) leftop).getBase());
				ArrayRef array_ref = arrayVariables.get(r0);
				ArrayLocal array_loc = arrayLocals.get(r0);
				if (array_ref != null) {
					GlobalVar array = getGlobalVar(array_ref.getArray(),
							inValue.getAtomicMethod());
					if (((JArrayRef) leftop).getIndex() instanceof JimpleLocal) {
						LocalVar index = new LocalVar(thisMethod,
								(JimpleLocal) ((JArrayRef) leftop).getIndex());
						addDepEdge(index, array, outValue, stmt);
					}
					if (rightop instanceof JimpleLocal) {
						Variable right = new LocalVar(thisMethod,
								(JimpleLocal) rightop);
						addDepEdge(right, array, outValue, stmt);
					} else if (!(rightop instanceof Constant)) {
						System.err
								.println("Dep1.IntraAnalysis: (1) What else? "
										+ rightop + "(" + rightop.getClass()
										+ ")");
						System.err.println(unit + "\n");
					}
					addDepEdge(r0, array, outValue, stmt);
				} else if (array_loc != null) {
					LocalVar array = new LocalVar(array_loc.getArray());
					if (((JArrayRef) leftop).getIndex() instanceof JimpleLocal) {
						LocalVar index = new LocalVar(thisMethod,
								(JimpleLocal) ((JArrayRef) leftop).getIndex());
						addDepEdge(index, array, outValue, stmt);
					}
					if (rightop instanceof JimpleLocal) {
						Variable right = new LocalVar(thisMethod,
								(JimpleLocal) rightop);
						addDepEdge(right, array, outValue, stmt);
					} else if (!(rightop instanceof Constant)) {
						System.err
								.println("Dep1.IntraAnalysis: (2) What else? "
										+ rightop + "(" + rightop.getClass()
										+ ")");
					}
					addDepEdge(r0, array, outValue, stmt);
				} else {
				}// Non important array
			}
			/**
			 * "$r1 = $r0[$i0]": - level 0? If yes: $r1 <-- array($r0) - $r1 <--
			 * $i0 - $r1 <-- dep($r0)
			 */
			else if (rightop instanceof JArrayRef) {
				ArrayRef array_ref = arrayVariables.get(new LocalVar(
						thisMethod, (JimpleLocal) ((JArrayRef) rightop)
								.getBase()));
				ArrayLocal array_loc = arrayLocals.get(new LocalVar(thisMethod,
						(JimpleLocal) ((JArrayRef) rightop).getBase()));
				if (array_ref != null) {
					Variable left = new LocalVar(thisMethod,
							(JimpleLocal) leftop);
					int level = array_ref.getLevel();
					if (level == 1) {
						GlobalVar array = getGlobalVar(array_ref.getArray(),
								inValue.getAtomicMethod());
						addDepEdge(array, left, outValue, stmt);// $r1 <--
																// array($r0)
					} else {
						arrayVariables.put(new LocalVar(thisMethod,
								(JimpleLocal) leftop),
								new ArrayRef(array_ref.getArray(), level - 1));
					}
					// $r1 <-- $i0
					Value aux = ((JArrayRef) rightop).getIndex();
					if (aux instanceof JimpleLocal) {
						LocalVar index = new LocalVar(thisMethod,
								(JimpleLocal) aux);
						addDepEdge(index, left, outValue, stmt);// $r1 <-- $i0
					}
					LocalVar r0 = new LocalVar(thisMethod,
							(JimpleLocal) ((JArrayRef) rightop).getBase());
					addDepEdge(r0, left, outValue, stmt);// $r1 <-- dep($r0)
				} else if (array_loc != null) {
					Variable left = new LocalVar(thisMethod,
							(JimpleLocal) leftop);
					int level = array_loc.getLevel();
					if (level == 1) {
						Variable array = new LocalVar(array_loc.getArray());
						addDepEdge(array, left, outValue, stmt); // $r1 <--
																	// array($r0)
					} else {
						arrayLocals
								.put(new LocalVar(thisMethod,
										(JimpleLocal) leftop), new ArrayLocal(
										array_loc.getArray(), level - 1));
					}
					// $r1 <-- $i0
					Value aux = ((JArrayRef) rightop).getIndex();
					if (aux instanceof JimpleLocal) {
						LocalVar index = new LocalVar(thisMethod,
								(JimpleLocal) aux);
						addDepEdge(index, left, outValue, stmt);// $r1 <-- $i0
					}
					LocalVar r0 = new LocalVar(thisMethod,
							(JimpleLocal) ((JArrayRef) rightop).getBase());
					// $r1 <-- dep($r0)
					addDepEdge(r0, left, outValue, stmt);// $r1 <-- dep($r0)
				} else {
				}// Non Important array
			}
			
			// rightVar is a global variable
			else if(leftop instanceof FieldRef){
				GlobalVar left = getGlobalVar((FieldRef) leftop,
						inValue.getAtomicMethod());
				outValue.setHashCode(new VarVersion(left, getStmtHashCode(stmt)));
				if (rightop instanceof JimpleLocal) {
					Variable right = new LocalVar(thisMethod, (JimpleLocal) rightop);
					addDepEdge(right, left, outValue, stmt);
				}
				else if(rightop instanceof Constant){}
			}
			
			// leftVar is a local variable
			else if(leftop instanceof JimpleLocal){
				LocalVar left = new LocalVar(thisMethod,
						(JimpleLocal) leftop);
				outValue.setHashCode(new VarVersion(left, getStmtHashCode(stmt)));
				
				// this local as no longer the old globalVariable's value
				sameObjects.remove(left);
				
				if(rightop instanceof JimpleLocal) {
					Variable right = new LocalVar(thisMethod, (JimpleLocal) rightop);
					addDepEdge(right, left, outValue, stmt);
				}
				else if (rightop instanceof FieldRef) {
					Type type = ((FieldRef) rightop).getType();
					LocalVar local = new LocalVar(thisMethod, (JimpleLocal) leftop);
					FieldRef global = (FieldRef) rightop;
					// If it is an array store it in the map
					if (type != null && type.toString().contains("[]")) {
						arrayVariables.put(local, new ArrayRef(global,
								countBrackets(type.toString())));
					}
					// Otherwise, add this dependency
					else {
						GlobalVar right = getGlobalVar((FieldRef) rightop,
								inValue.getAtomicMethod());
						addDepEdge(right, left, outValue, stmt);
						if (!isPrimitive((JimpleLocal) leftop)) {
//							addDepEdge(left, right, outValue, stmt, true);
							sameObjects.put(local, right);
						}
					}
				}
				// local = ? +|x|/|-|... ?
				else if (rightop instanceof AbstractBinopExpr) {
					AbstractBinopExpr bin_exp = (AbstractBinopExpr) rightop;
					Value value1 = bin_exp.getOp1();
					Value value2 = bin_exp.getOp2();
					if (value1 instanceof JimpleLocal) {
						Variable v1 = new LocalVar(thisMethod, (JimpleLocal) value1);
						addDepEdge(v1, left, outValue, stmt);
					}
					if (value2 instanceof JimpleLocal) {
						Variable v2 = new LocalVar(thisMethod, (JimpleLocal) value2);
						addDepEdge(v2, left, outValue, stmt);
					} else if (!(value1 instanceof JimpleLocal || value1 instanceof Constant)
							|| !(value2 instanceof JimpleLocal || value2 instanceof Constant)) {

						System.err
								.println("Dep1.IntraAnalysis: ERROR! Parameters '"
										+ value1 + "'(" + value1.getClass()
										+ ") and '" + value2 + "'("
										+ value2.getClass()
										+ ") should be JimpleLocal");
					}
				}
				else if (rightop instanceof AbstractUnopExpr) {
					Value v = ((AbstractUnopExpr) rightop).getOp();
					if (v instanceof JimpleLocal) {
						LocalVar right = new LocalVar(thisMethod, (JimpleLocal) v);
						ArrayRef array = arrayVariables.get(right);
						if (array != null) {
							GlobalVar glo = getGlobalVar(array.getArray(),
									inValue.getAtomicMethod());
							addDepEdge(glo, left, outValue, stmt);
						} else {
							addDepEdge(right, left, outValue, stmt);
						}
					} else {
						System.err.println("Dep1.IntraAnalysis, AbstractUnopExpr :" +
								" Type is not JimpleLocal " + v + "(" + v.getClass() + ")");
					}
				}
				else if (rightop instanceof JCastExpr) {
					JCastExpr cast = (JCastExpr) rightop;
					Value op = cast.getOp();
					Variable right = null;
					if (op instanceof JimpleLocal) {
						right = new LocalVar(thisMethod, (JimpleLocal) op);
					} else if (op instanceof FieldRef) {
						right = getGlobalVar((FieldRef) op,
								inValue.getAtomicMethod());
					}
					if (right != null) {
						addDepEdge(right, left, outValue, stmt);
					}
				}
				// vector = int[$i1][$i2][..
				else if (rightop instanceof JNewMultiArrayExpr) {
					JNewMultiArrayExpr expr = (JNewMultiArrayExpr) rightop;
					arrayLocals.put(left,
							new ArrayLocal(left, expr.getSizeCount()));
					for (int i = 0; i < expr.getSizeCount(); i++) {
						Value index_i = expr.getSize(i);
						if (index_i instanceof JimpleLocal) {
							Variable child = new LocalVar(thisMethod,
									(JimpleLocal) index_i);
							addDepEdge(child, left, outValue, stmt);
						} else if (!(index_i instanceof Constant)) {
							System.err.println("Dep1.IntraAnalysis: (4) What else?"
											+ index_i+ "("+ index_i.getClass() + ")");
						}
					}
				}
			}
			// Other JAssignStmt Struture
			else if(!(rightop instanceof JNewExpr) && !(rightop instanceof JCastExpr) &&
					!(rightop instanceof AbstractInstanceOfExpr) &&
					!(rightop instanceof JArrayRef) &&
					!(rightop instanceof Constant)){
				System.err.println("\n(Dep1.IntraAnalysis)Other JAssignStmt Struture:");
				System.err.println("Left = " + leftop.getClass() + "("+ leftop + ")");
				System.err.println("Right = " + rightop.getClass() + "("+ rightop + ")\n");
			}
		} else if (unit instanceof JReturnStmt) {
			dumpArrayVariables(outValue, inValue.getAtomicMethod());
			Value return_var = ((JReturnStmt) unit).getOp();
			if (return_var instanceof JimpleLocal) {
				LocalVar local = new LocalVar(thisMethod,
						(JimpleLocal) return_var);
				ReturnVar var = new ReturnVar(thisMethod);
				addDepEdge(local, var, outValue, stmt);
			} else if (!(return_var instanceof Constant)) {
				System.err
						.println("Dep1.IntraAnalysis: ReturnType shouldn't be a JimpleLocal or a Constant? "
								+ return_var
								+ "("
								+ return_var.getClass()
								+ ")");
			}
		} else if (unit instanceof JReturnVoidStmt) {
			dumpArrayVariables(outValue, inValue.getAtomicMethod());
		} else if (unit instanceof JIdentityStmt) {
			String unitStr = unit.toString();
			Value value = ((JIdentityStmt) unit).getLeftOp();
			if (!thisMethod.getName().contains("<init>")
					&& unitStr.contains("@parameter")) {
				if (value instanceof JimpleLocal) {
					JimpleLocal local = (JimpleLocal) value;
					int pos = unitStr.indexOf("@parameter") + 10;
					int pos2 = unitStr.indexOf(" ", pos);
					String num = unitStr.substring(pos, pos2 - 1);
					try {
						int paramNum = Integer.parseInt(num);
						LocalVar localVar = new LocalVar(thisMethod, local);
						ParamVar paramVar = new ParamVar(thisMethod, paramNum);
						addDepEdge(paramVar, localVar, outValue, stmt);
					} catch (NumberFormatException e) {
						System.err
								.println("Dep1.IntraAnalysis: Error! (Shouldn't happen) - "
										+ unit + ", num = " + num);
						e.printStackTrace();
					}
				} else {
					System.err
							.println("Dep1.IntraAnalysis: What could be leftop's type? "
									+ "method = "
									+ thisMethod
									+ ", value = "
									+ value + "(" + value.getClass() + ")");
				}
			}
		} else if (unit instanceof JIfStmt) {
			JIfStmt ifStmt = (JIfStmt) unit;
			List<?> usedVars = ifStmt.getUseBoxes();
			for (Object obj : usedVars) {
				if (obj instanceof ImmediateBox) {
					Value value = ((ImmediateBox) obj).getValue();
					if (value instanceof JimpleLocal) {
						LocalVar loc_var = new LocalVar(thisMethod,
								(JimpleLocal) value);
						IfConditionVar if_var = new IfLocalVar(thisMethod,
								(JimpleLocal) value, ifStmt);
						addDepEdge(loc_var, if_var, outValue, stmt);
					} else if (value instanceof FieldRef) {
						GlobalVar glo_var = getGlobalVar((FieldRef) value,
								inValue.getAtomicMethod());
						IfConditionVar if_var = new IfGlobalVar(thisMethod,
								glo_var, ifStmt);
						addDepEdge(glo_var, if_var, outValue, stmt);
					} else if (!(value instanceof Constant)) {
						System.err
								.println("Dep1.IntraAnalysis: Value is not FieldRef "
										+ "or Local : "
										+ value
										+ " "
										+ value.getClass());
						break;
					}
				}
			}
		} else if (unit instanceof JThrowStmt) {
		}// Throw exception with localVariable (e.g. throw $r0)
		else if (unit instanceof JGotoStmt) {
		}// JGotoStmt used, for instances, in loops
		else if (unit instanceof JTableSwitchStmt) {
		}// JTableSwitchStmt used
		else if (unit instanceof MonitorStmt) {
		}// Monitor Statement
		else if (unit instanceof JLookupSwitchStmt) {
		}// JLookupSwitchStmt used
		// Others..
		else {
			System.err.println("(" + thisMethod.getName()
					+ ")Other class : (in " + unit.getClass() + ") " + unit);
		}
		// System.out.println( unit +
		// "("+unit.getClass()+") - \n"+inValue+"\n\n"+outValue);
	}

	/**
	 * Adds a new dependency to the Lattice.
	 * 
	 * @param origin
	 *            origin of the dependency/edge
	 * @param dest
	 *            destination of the dependency/edge
	 * @param outValue
	 *            Lattice in which we are adding the dependency/edge
	 * @param stmt
	 *            statement that generated this edge
	 * @param generatedByAssignment
	 *            just say if this dependency was generated automaticaly. This happens
	 *            when we have "$r1 = var" and the type is not primitive. In these cases, we put
	 *            a "var -> r1", and an automatic dependency "r1 -> var" (not updating var's hashcode) 
	 */
	public void addDepEdge(Variable origin, Variable dest,
		GraphLattice outValue, Stmt stmt, boolean generatedByAssignment) {
		Set<Integer> hashCodes = outValue.getHashCodes(origin);
		
		Set<VarVersion> destSet = createDestNode(dest,stmt,outValue,generatedByAssignment);
		VarVersion origin_node;
		for (Integer i : hashCodes) {
			origin_node = new VarVersion(origin, i);
			// Add Edge
			for(VarVersion dest_node : destSet){
				outValue.add(new DepEdge(origin_node, dest_node, false));
			}
		}
	}
	
	public void addDepEdge(Variable origin, Variable dest,
			GraphLattice outValue, Stmt stmt) {
		addDepEdge(origin,dest,outValue,stmt,true);
	}

	/**
	 * Creates the GraphNode object for the destination of this dependency.
	 * Adds its new hashcode in the lattice
	 */
	private Set<VarVersion> createDestNode(Variable dest, Stmt stmt, GraphLattice outValue, boolean generatedByAssignment) {
		Set<VarVersion> set = new HashSet<VarVersion>();
		// If it is a localVar, may be it is a reference to a globalVar
		if(dest instanceof LocalVar && sameObjects.containsKey(dest)){
			GlobalVar global = sameObjects.get(dest);
			Set<Integer> globalHashCodes = outValue.getHashCodes(global);
			for(int hash : globalHashCodes)
				set.add(new VarVersion(global,hash));
		}
		if(dest instanceof ParamRef || dest instanceof ParamVar || 
			dest instanceof ReturnRef || dest instanceof ReturnVar || 
			dest instanceof IfConditionVar){
			set.add(new VarVersion(dest,0));
			return set;
		}
		if(!generatedByAssignment){
			for(int i : outValue.getHashCodes(dest)){
				set.add(new VarVersion(dest,i));
			}
			return set;
		}
		VarVersion dest_node = new VarVersion(dest, getStmtHashCode(stmt)); 
		// Add new destination's hashCode
		outValue.setHashCode(dest_node);
		set.add(dest_node);
		return set;
	}

	/**
	 * Now, it just calls the hashCode method of the soot object, but in the
	 * future it can be more complex function.
	 */
	private int getStmtHashCode(Stmt stmt) {
		return stmt == null? GraphLattice.SPECIAL_HASH : stmt.hashCode();
	}

	/**
	 * Returns if this (local) variable has a primitive type
	 */
	private boolean isPrimitive(JimpleLocal local) {
		Type type = local.getType();
		return type instanceof PrimType;
	}

	/**
	 * Links up every argument/parameter with the methods baseVar and the
	 * leftVar. We supose that the methods are "modifiers" without return, or
	 * "getters" that dont modify the object. The baseVar is only affected in
	 * the first group of methods
	 * 
	 * @param args - arguments of this method call
	 * @param baseVar - variable where the method was called
	 * @param leftVar - left variable of the assign statement
	 * @param outValue - Lattice in this statement
	 * @param stmt - invoke statement
	 */
	private void linkupArgsAndObject(Iterator<Value> args, LocalVar baseVar,
			LocalVar leftVar, GraphLattice outValue, Stmt stmt) {
		if (baseVar == null && leftVar == null) {
			return;
		}
		int argNum = 0;
		while (args.hasNext()) {
			Value arg = args.next();
			if (arg instanceof JimpleLocal) {
				LocalVar lvar = new LocalVar(thisMethod, (JimpleLocal) arg);
				if (leftVar != null) {
					addDepEdge(lvar, leftVar, outValue, stmt);
				}
				else if (baseVar != null) {
					addDepEdge(lvar, baseVar, outValue, stmt, true);
				}
			} else if (!(arg instanceof Constant)) {
				System.err
						.println("Dep1.IntraAnalysis: Warning: Arg should be JimpleLocal (or constant)"
								+ "arg = " + arg + "(" + arg.getClass() + ")");
			}
			argNum++;
		}
	}

	/**
	 * Link up a method's arguments with its localVars Imagine we have:
	 * "public ? method(Type x1,Type x2,Type x3...){...}" and we called it with
	 * "method(a1,a2,a3..)"
	 * 
	 * locals = [x1,x2,x3...,xn, l1,l2,l3...] (l - local variables of the
	 * method) args = [a1,a2,a3...]
	 * 
	 */
	private void linkupArgsAndLocals(Stmt stmt, SootMethod method,
			Iterator<Value> args, GraphLattice outValue) {
		if (!method.hasActiveBody() || method.getSignature().contains("<init>")) {
			return;
		}
		Iterator<Local> locals = method.getActiveBody().getLocals().iterator();
		int argNum = 0;
		while (args.hasNext()) {
			if (!locals.hasNext()) {
				System.err.println("Dep1.IntraAnalysis: Shouldn't happen!! "
						+ "locals > args. locals = "
						+ method.getActiveBody().getLocals() + ", args = "
						+ args);
			} else {
				Value arg = args.next();
				Local local = locals.next();
				if (local instanceof JimpleLocal) {
					ParamRef pvar = new ParamRef(thisMethod, argNum, stmt);
					if (arg instanceof JimpleLocal) {
						LocalVar lvar = new LocalVar(thisMethod,
								(JimpleLocal) arg);
						addDepEdge(lvar, pvar, outValue, stmt);
					} else if (!(arg instanceof Constant)) {
						System.err
								.println("Dep1.IntraAnalysis: Warning: Arg should be JimpleLocal (or constant)"
										+ "arg = "
										+ arg
										+ "("
										+ arg.getClass()
										+ ")");
					}
				} else {
					System.err
							.println("Dep1.IntraAnalysis: Warning: Local should be a JimpleLocal "
									+ "local = "
									+ local
									+ "("
									+ local.getClass() + ")");
				}
				argNum++;
			}
		}
	}

	private void dumpArrayVariables(GraphLattice outValue, SootMethod atomic) {
		for (Entry<LocalVar, ArrayRef> entry : arrayVariables.entrySet()) {
			GlobalVar glo = getGlobalVar(entry.getValue().getArray(), atomic);
			addDepEdge(glo, entry.getKey(), outValue, null);
		}
	}

	/**
	 * Return the "base" of an invoke expression like "$r1 = base.method()";
	 */
	private JimpleLocal getInvokeExprLocal(InvokeExpr expr) {
		if (expr instanceof JVirtualInvokeExpr
				&& ((JVirtualInvokeExpr) expr).getBase() instanceof JimpleLocal) {
			return (JimpleLocal) ((JVirtualInvokeExpr) expr).getBase();
		} else if (expr instanceof JInterfaceInvokeExpr
				&& ((JInterfaceInvokeExpr) expr).getBase() instanceof JimpleLocal) {
			return (JimpleLocal) ((JInterfaceInvokeExpr) expr).getBase();
		} else if (expr instanceof JStaticInvokeExpr) {
			// Ignore! There is no local variable. Just a variable with the
			// class reference
		} else if (expr instanceof JSpecialInvokeExpr
				&& ((JSpecialInvokeExpr) expr).getBase() instanceof JimpleLocal) {
			return (JimpleLocal) ((JSpecialInvokeExpr) expr).getBase();
		} else {
			System.err.println("Other InvokeExpr class: " + (expr.getClass()));
		}
		return null;
	}

	/**
	 * Creates the GlobalVar being accessed. If this method is a non atomic
	 * called inside an atomic one, then the return will be an IndirectGlobalVar
	 * with the atomic method. Otherwise, the return will be a normal globalVar
	 * with thisMethod.
	 * 
	 * @param ref
	 *            - Variable being accessed
	 * @param thisMethod
	 *            - method where the access as been made
	 * @param atomic
	 *            - Atomic method that called this method or "null" otherwise
	 */
	private GlobalVar getGlobalVar(FieldRef ref, SootMethod atomic) {
		if (!MethodUtils.methodIsAtomic(thisMethod) && atomic != null) {
			return new GlobalVar(ref, atomic, false);
		}
		return new GlobalVar(ref, thisMethod, true);
	}

	private void treateNativeDependencies(SootMethod nativeMethod,
			SootMethod atomic, GraphLattice outValue, Stmt stmt) {
		if (atomic == null) {
			return;
		}
		InvokeExpr expr = stmt.getInvokeExpr();
		@SuppressWarnings("unchecked")
		Iterator<Value> args = expr.getArgs().iterator();
		int argNum = 0;
		while (args.hasNext()) {
			Value arg = args.next();
			if (arg instanceof JimpleLocal) {
				LocalVar lvar = new LocalVar(thisMethod, (JimpleLocal) arg);
				NativeGlobalVar nativeVar = new NativeGlobalVar(nativeMethod,
						thisMethod, atomic);
				addDepEdge(lvar, nativeVar, outValue, stmt);
			} else if (!(arg instanceof Constant)) {
				System.err
						.println("Dep1.IntraAnalysis: Warning: Arg should be JimpleLocal (or constant)"
								+ "arg = " + arg + "(" + arg.getClass() + ")");
			}
			argNum++;
		}
	}

	private Integer countBrackets(String string) {
		char[] str = string.toCharArray();
		int num_brackets = 0;
		for (int i = 0; i < str.length - 1; i++) {
			if (str[i] == '[' && str[i + 1] == ']') {
				num_brackets++;
				i++;
			}
		}
		return num_brackets;
	}

	protected void copy(GraphLattice source, GraphLattice dest) {
		source.copy(dest);
	}

	@Override
	protected GraphLattice entryInitialFlow() {
		return entryLattice;
	}

	@Override
	protected void merge(GraphLattice in1, GraphLattice in2, GraphLattice out) {
		in1.union(in2, out);
	}

	@Override
	protected GraphLattice newInitialFlow() {
		return new GraphLattice();
	}

	public GraphLattice getFlowAtEnd() {
		GraphLattice res = newInitialFlow();
		// merge the flowsets of all the method's exit-points
		List<Unit> tails = this.graph.getTails();
		for (Unit unit : tails) {
			GraphLattice sep1 = getFlowAfter(unit);
			if (sep1 == null) { // this method was not analyzed
				return res;
			}
			GraphLattice tmp = newInitialFlow();
			sep1.union(res, tmp);
			res = tmp;
		}
		return res;
	}
}
