package pt.moth.instanceTypeAnalysis;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import pt.moth.util.variables.GlobalVar;


/**
 * This class allows us to store every global field that has implementing
 * classes. A "result" is represented by a variable (that has the global field)
 * and a set of classes that implement that interface that may be the
 * implementing class
 * 
 * These results are stored by the First phase of the Instance Type Analysis and
 * will be used in the Second phase, associating each InvokeStatement to its
 * possible implementing classes
 * 
 * @author Vasco Pessanha
 * 
 */
public class FieldClassesCollector {
	private static Map<GlobalVar, Set<String>> results = new HashMap<GlobalVar, Set<String>>();

	public static void putResult(GlobalVar v, String implementingClass) {
		// System.out.println("PUT RESULT = "+v+" => " +implementingClass);
		Set<String> classes = results.get(v);
		if (classes == null)
			classes = new HashSet<String>();
		classes.add(implementingClass);
		results.put(v, classes);
	}

	public static void putResult(GlobalVar v, Set<String> newClasses) {
		// System.out.println("PUT RESULTS = "+v+" => " +newClasses);
		Set<String> classes = results.get(v);
		if (classes == null)
			classes = new HashSet<String>();
		for (String s : newClasses)
			classes.add(s);
		results.put(v, classes);
	}

	public static Set<String> getImplementingClasses(GlobalVar v) {
		return results.get(v);
	}
}
