package pt.moth.instanceTypeAnalysis.firstPhase;


import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import pt.moth.instanceTypeAnalysis.FieldClassesCollector;
import pt.moth.util.Pair;
import pt.moth.util.variables.GlobalVar;
import pt.moth.util.variables.LocalVar;


import soot.SootMethod;
import soot.Unit;
import soot.Value;
import soot.jimple.Constant;
import soot.jimple.FieldRef;
import soot.jimple.MonitorStmt;
import soot.jimple.StaticFieldRef;
import soot.jimple.Stmt;
import soot.jimple.internal.AbstractBinopExpr;
import soot.jimple.internal.AbstractInstanceOfExpr;
import soot.jimple.internal.AbstractUnopExpr;
import soot.jimple.internal.JAddExpr;
import soot.jimple.internal.JArrayRef;
import soot.jimple.internal.JAssignStmt;
import soot.jimple.internal.JCastExpr;
import soot.jimple.internal.JGotoStmt;
import soot.jimple.internal.JIdentityStmt;
import soot.jimple.internal.JIfStmt;
import soot.jimple.internal.JInstanceFieldRef;
import soot.jimple.internal.JLookupSwitchStmt;
import soot.jimple.internal.JNewArrayExpr;
import soot.jimple.internal.JNewExpr;
import soot.jimple.internal.JNewMultiArrayExpr;
import soot.jimple.internal.JReturnStmt;
import soot.jimple.internal.JReturnVoidStmt;
import soot.jimple.internal.JTableSwitchStmt;
import soot.jimple.internal.JThrowStmt;
import soot.jimple.internal.JimpleLocal;
import soot.toolkits.graph.DirectedGraph;
import soot.toolkits.graph.UnitGraph;
import soot.toolkits.scalar.ForwardFlowAnalysis;

public class IntraAnalysis extends ForwardFlowAnalysis<Unit, ClassLattice>{

	protected InterAnalysis interAnalysis;
	protected SootMethod thisMethod;
	
	/**
	 * This structure allows us to keep the set of readVariables
	 * that are stored in the outValue Lattice in the Return Stmt
	 */
	protected Map<LocalVar, FieldRef> readVariables;
	protected Map<LocalVar, String> newsExpressions;
	
	public IntraAnalysis(DirectedGraph<Unit> graph, ClassLattice paramValues, InterAnalysis interAnalysis) {
		super(graph);
		this.interAnalysis = interAnalysis;
		this.thisMethod = ((UnitGraph) graph).getBody().getMethod();

		this.readVariables = new HashMap<LocalVar, FieldRef>();
		this.newsExpressions = new HashMap<LocalVar, String>();
		doAnalysis();
	}
	
	@Override
	protected void flowThrough(ClassLattice inValue, Unit unit, ClassLattice outValue) {
		inValue.copy(outValue);		
		if (((Stmt) unit).containsInvokeExpr()) {
			interAnalysis.handleMethodInvocation(thisMethod, new ClassLattice(), (Stmt) unit);
		}
		// This instruction is an assignment x = 4 or z = x
		else if (unit instanceof JAssignStmt) {
			Value leftop = ((JAssignStmt) unit).getLeftOp();
			Value rightop = ((JAssignStmt) unit).getRightOp();
			
			// global = $r0 ($r0 has implementing classes)
			if (leftop instanceof JInstanceFieldRef || leftop instanceof StaticFieldRef) {
				if(rightop instanceof JimpleLocal){
					Set<String> newClasses = inValue.getImplementations(new LocalVar(thisMethod,(JimpleLocal) rightop));
					if(newClasses != null && newClasses.size() > 0){
						GlobalVar v = new GlobalVar((FieldRef)leftop,true);
						
						FieldClassesCollector.putResult(v, newClasses);
//						System.out.println("PUT CLASS ("+v+","+newClass+")");
					}
				}
			}
			else if(leftop instanceof JArrayRef){}
			else if (leftop instanceof JimpleLocal){
				if(rightop instanceof JNewExpr){
					JNewExpr expr = (JNewExpr)rightop;
					LocalVar local = new LocalVar(thisMethod,(JimpleLocal)leftop);
					outValue.add(new Pair<LocalVar,String>(local,expr.getType().toString()));
				}
				else if(rightop instanceof JInstanceFieldRef){} //Ignorar statements tipo "local = global"
				else if(rightop instanceof StaticFieldRef){} //Ignorar statements tipo "local = Class.x"
				else if(rightop instanceof JCastExpr){}		//Ignorar casts
				else if(rightop instanceof JAddExpr){}	//Ignora somas..
				else if(rightop instanceof JimpleLocal){}	//Ignorar local = local...
				else if(rightop instanceof Constant){}	//Ignorar constantes
				else if(rightop instanceof JNewArrayExpr){}		//Ignorar new ?[]
				else if(rightop instanceof AbstractUnopExpr){}	//Ignorar expressoes de um arg (e.g. $r0.length())
				else if(rightop instanceof AbstractBinopExpr){}	//Ignorar expressoes de 2args (e.g. cmp)
				else if(rightop instanceof AbstractInstanceOfExpr){} //Ignorar expr "instanceof"
				else if(rightop instanceof JArrayRef){}				//Ignorar expressoes de arrays
				else if(rightop instanceof JNewMultiArrayExpr){}	//Ignorar New de arrys multiplos (tds duma vez)
				else{
					System.err.println("\nOther JAssignStmt Struture:");
					System.err.println("Left = " + leftop.getClass() + "("+ leftop + ")");
					System.err.println("Right = " + rightop.getClass() + "("+ rightop + ")\n");
				}
			}
			// Other JAssignStmt Struture
			else {
				System.err.println("\nOther JAssignStmt Struture:");
				System.err.println("Left = " + leftop.getClass() + "("+ leftop + ")");
				System.err.println("Right = " + rightop.getClass() + "("+ rightop + ")\n");
			}
		}
		else if(unit instanceof JReturnStmt || unit instanceof JReturnVoidStmt){}
		else if(unit instanceof JIdentityStmt){}
		else if(unit instanceof JIfStmt){}//This can identify a if with local vars (e.g. if($r0 == null))
		else if(unit instanceof JThrowStmt){}//Throw exception with localVariable (e.g. throw $r0)
		else if(unit instanceof JGotoStmt){}//JGotoStmt used, for instances, in loops
		else if(unit instanceof JTableSwitchStmt){}//JTableSwitchStmt used
		else if(unit instanceof MonitorStmt){}//Monitor Statement
		else if(unit instanceof JLookupSwitchStmt){}//JLookupSwitchStmt used
		// Others..
		else {
			System.err.println("("+thisMethod.getName()+")Other class : (in "+unit.getClass()+") "+unit);
		}
//		System.out.println("("+thisMethod.getDeclaringClass()+","+thisMethod.getName() + ") " + unit + "("+unit.getClass()+") - "+inValue+","+outValue);
	}
	
	protected void copy(ClassLattice source, ClassLattice dest) {
		source.copy(dest);
	}

	@Override
	protected ClassLattice entryInitialFlow() {
		return new ClassLattice();
	}

	@Override
	protected void merge(ClassLattice in1, ClassLattice in2, ClassLattice out) {
		in1.union(in2, out);
	}

	@Override
	protected ClassLattice newInitialFlow() {
		return new ClassLattice();
	}

	public ClassLattice getFlowAtEnd() {
		ClassLattice res = newInitialFlow();

		// merge the flowsets of all the method's exit-points
		List<Unit> tails = this.graph.getTails();
		for (Unit unit : tails) {
			ClassLattice sep1 = getFlowAfter(unit);
			if (sep1 == null) { // this method was not analyzed
				return res;
			}
			ClassLattice tmp = newInitialFlow();
			sep1.union(res, tmp);
			res = tmp;
		}
		return res;
	}
}
