package pt.moth.processAnalysis;

import java.util.HashSet;
import java.util.Set;
import soot.SootMethod;

/**
 * This class represent a process with its name, description, and atomic blocks.
 * 
 * @author Vasco Pessanha
 */
public class Process {
	//Name of the process.. Generaly it's just "Process X"
	protected String name;
	// A little description of the process. Generaly it has the name of the
	// method that creates the process (Class.run())
	protected String description;
	//Set of atomicmethods called by this process
	protected Set<SootMethod> methods;
	
	public Process(String name, String description){
		this.name = name;
		this.description = description;
		methods = new HashSet<SootMethod>();
	}
	public Process(Process p){
		this.name = p.name;
		this.description = p.description;
		methods = new HashSet<SootMethod>();
		for(SootMethod m : p.getMethods())
			methods.add(m);
	}
	public Process(String name){
		this(name,"");
	}
	
	public boolean addMethod(SootMethod method){
		return methods.add(method);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((methods == null) ? 0 : methods.hashCode());
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Process other = (Process) obj;
		if (methods == null) {
			if (other.methods != null)
				return false;
		} else if (!methods.equals(other.methods))
			return false;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		return true;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}
	
	public String getDescription() {
		return description;
	}
	
	public void setDescription(String description) {
		this.description = description;
	}
	
	public Set<SootMethod> getMethods() {
		return methods;
	}

	public void setMethods(Set<SootMethod> methods) {
		this.methods = methods;
	}
	
	public Process clone(){
		return new Process(this);
	}
	
	public String toString(){
		String result = name+" ("+description+")\nMethods:\n";
		for(SootMethod m : methods){
			result += m+"\n";
		}
		return result;
	}
}
