package pt.moth.sensors;

import java.util.HashSet;
import java.util.List;
import java.util.Map.Entry;
import java.util.Set;

import pt.moth.dependencies.Dep;
import pt.moth.dependencies.thirdPhase.ViewDepCollector;
import pt.moth.processAnalysis.Process;
import pt.moth.processAnalysis.ProcessGenerator;
import pt.moth.util.View;
import pt.moth.util.conflicts.Conflict;
import pt.moth.util.conflicts.DepConflict;
import pt.moth.util.variables.GlobalVar;
import pt.moth.viewsAnalysis.ViewsCollector;



public class DepSensor implements Sensor {
	
	private Set<Entry<String, List<View>>> process_w_views;
	private Set<Dep> dependencies;
	
	public DepSensor(){
		 process_w_views = new HashSet<Entry<String,List<View>>>();
		 dependencies = new HashSet<Dep>();
	}
	
	@Override
	public void pushViewInformation() {
		process_w_views = ViewsCollector.getWriteViews();
	}

	@Override
	public void pushDepInformation() {
		dependencies = ViewDepCollector.getDependencies();
	}

	@Override
	public Set<Conflict> getConflicts() {
		Set<Conflict> result = new HashSet<Conflict>();
		for(Dep d : dependencies){
			GlobalVar var = d.getDepVariable();
			for(Entry<String, List<View>> e : process_w_views){
				Set<View> w_views = getWritingViews(e.getValue(),var);
				for(View view : w_views){
					Process p1 = d.getProcess();
					Process p2 = ProcessGenerator.getProcess(e.getKey());
					assert(p2 != null);
					result.add(new DepConflict(p1,p2,d,view));
				}
			}
		}
		return result;
	}

	/**
	 * Return all the views of "views" that write on "var"
	 */
	private Set<View> getWritingViews(List<View> views, GlobalVar var) {
		Set<View> result = new HashSet<View>();
		for(View v : views){
			if(v.writesOn(var)){
				result.add(v);
			}
		}
		return result;
	}
}
