package pt.moth.util;

import java.util.HashSet;
import java.util.Set;

import pt.moth.util.variables.GlobalVar;
import pt.moth.util.variables.Variable;
import pt.moth.viewsAnalysis.Lattice;

import soot.SootMethod;

public class View {
	protected Lattice accesses;
	protected SootMethod method;
	
	public View (Lattice accesses, SootMethod method){
		this.accesses = accesses;
		this.method = method;
	}
	public Lattice getAccesses() {
		return accesses;
	}
	public void setAccesses(Lattice accesses) {
		this.accesses = accesses;
	}
	public SootMethod getMethod() {
		return method;
	}
	public void setMethod(SootMethod method) {
		this.method = method;
	}
	public boolean writesOn(GlobalVar v){
		for(IAccess a : accesses.toList()){
			if(a instanceof Access && v.equalsIgnoreMethod((GlobalVar)((Access) a).getVariable()) && ((Access)a).isWrite()){
				return true;
			}
		}
		return false;
	}
	public boolean readsFrom(GlobalVar v){
		for(IAccess a : accesses.toList()){
			if(a instanceof Access && v.equalsIgnoreMethod((GlobalVar)((Access) a).getVariable()) && ((Access)a).isRead()){
				return true;
			}
		}
		return false;
	}
	public boolean contains(View v){
		if(size() < v.size()){
			return false;
		}
		for(IAccess a : v.getAccesses().toList()){
			if(!this.accesses.contains(a)){
				return false;
			}
		}
		return true;
	}
	public boolean containsIgnoreType(View v){
		if(size() < v.size()){
			return false;
		}
		for(IAccess a : v.getAccesses().toList()){
			if(!this.accesses.containsIgnoreType(a)){
				return false;
			}
		}
		return true;
	}
	public int size(){
		return this.accesses.size();
	}
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((accesses == null) ? 0 : accesses.hashCode());
		result = prime * result + ((method == null) ? 0 : method.toString().hashCode());
		return result;
	}
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof View))
			return false;
		
		View other = (View) obj;
		return (accesses.equals(other.accesses)) &&
			(method.toString().equals(other.method.toString()));
	}
	public boolean equalsIgnoreType(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof View))
			return false;
		
		View other = (View) obj;
		return (accesses.equalsIgnoreCase(other.accesses)) &&
			(method.toString().equals(other.method.toString()));
	}
	public View intersection(View other){
		SootMethod m = this.method;
		Lattice lat = new Lattice();
		for(IAccess a : accesses.toList()){
			if(other.getAccesses().containsIgnoreType(a)){
				lat.add(a);
			}
		}
		return new View(lat, m);
	}
	public void copy(View v){
		v = new View((Lattice)accesses.clone(),method);
	}
	public String toString(){
		return method.getName() +" -> "+accesses.toString();
	}
	
	public View clone(){
		return new View(accesses,method);
	}
	public Set<Variable> getVariables() {
		Set<Variable> result = new HashSet<Variable>();
		for(IAccess a : accesses.toList()){
			result.add(a.getVariable());
		}
		return result;
	}
	
	/**
	 * Removes every access that accesses the variable "var"
	 * @param var - variable which accesses will be removed
	 */
	public void removeAccessesWithVar(Variable var) {
		Set<IAccess> toRemove = new HashSet<IAccess>();
		for(IAccess a : accesses.toList()){
			if(a.getVariable().equals(var)){
				toRemove.add(a);
			}
		}
		for(IAccess a : toRemove){
			accesses.remove(a);
		}
	}
	
	/**
	 * Returns true if this View accesses (reads from or writes in) the variable "v"
	 */
	public boolean accesses(Variable v) {
		for(IAccess a : accesses.toList()){
			if(a.getVariable().equals(v)){
				return true;
			}
		}
		return false;
	}
}
