package pt.moth.util.conflicts;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import pt.moth.util.View;
import pt.moth.util.variables.GlobalVar;


public class DepGroup implements IConflictGroup{

	protected List<DepConflict> conflicts;

	public DepGroup(DepConflict conflict) {
		conflicts = new ArrayList<DepConflict>();
		conflicts.add(conflict);
	}
	
	@Override
	public boolean addConflict(Conflict conflict) {
		if(!(conflict instanceof DepConflict)){
			return false;
		}
		DepConflict head = conflicts.get(0);
		DepConflict c = (DepConflict)conflict;
		if (head.getV1().equals(c.getV1()) && head.getV2().equals(c.getV2())){
//				&& head.getWriteView().equals(c.getWriteView())) {
			conflicts.add(c);
			return true;
		}
		return false;
	}
	
	@Override
	public String toString() {
		Iterator<DepConflict> it = conflicts.iterator();
		OneVarConflict head = (OneVarConflict) it.next();
		View v1 = head.getV1();
		View v2 = head.getV2();
		Set<GlobalVar> variables = new HashSet<GlobalVar>();
		Set<View> w_methods = new HashSet<View>();
		
		// Add head's variable and write method
		variables.add(head.getVar());
		w_methods.add(head.getWriteView());
		
		// Do the same for the rest of the conflicts
		while (it.hasNext()) {
			OneVarConflict c = it.next();
			variables.add(c.getVar());
			w_methods.add(c.getWriteView());
		}
		
		String result = "OneVarConflict!\nR = '" + v1.getMethod().getName()
				+ "()'\n" + " W = {";
		for(View v : w_methods){
			result += v.getMethod().getName()+", ";
		}
		result = result.substring(0, result.length()-2) + "}\n";
						
		result += "W = '" + v2.getMethod().getName() + "()'\nVariables = {";
		
		for(GlobalVar var : variables){
			result += var.getName()+", ";
		}
		return result.substring(0, result.length()-2) + "}\n";
	}
	/*
	@Override
	public String toString() {
		Iterator<DepConflict> it = conflicts.iterator();
		OneVarConflict head = (OneVarConflict) it.next();
		Set<String> varsTreated = new HashSet<String>(); 
		String result = "OneVarConflict between" + "!\nR = '" + head.getV1().getMethod().getName()
				+ "()'\n" + " W = '" + head.getWriteView().getMethod().getName() + "()'\nW = '"
				+ head.getV2().getMethod().getName() + "()'\nVariables = {"
				+ head.getVar().getName();
		varsTreated.add(head.getVar().getName());
		while (it.hasNext()) {
			String var = ((OneVarConflict) it.next()).getVar().getName();
			if(!varsTreated.contains(var)){
				varsTreated.add(var);
				result += ", " + var;
			}
		}
		return result + "}";
	}
	*/
}
