package pt.moth.util.graphs;

import pt.moth.dependencies.VarVersion;

/**
 * This class represents a graph's Dependency Edge and extends the normal Edge
 *  Class (Edge<E>). This special edges allows us to distinguish if a this
 *  dependency is a ControlDep or a DataDep. A DataDep is a dependency between
 *  two variable, where the second variable's value depends on the first's. A
 *  ControlDep ("A --> B") is generated when we have a cenario like: 
 * 
 * if(Read(A)){
 * 	Write(B);
 * }
 * 
 * @author Vasco Pessanha
 */
public class DepEdge extends Edge<VarVersion> {

	
	// 0 - Normal Data Dependency, 1 - If/Control dependency 
	protected boolean isControlDep;

	public DepEdge(VarVersion origin, VarVersion dest, boolean isControlDep) {
		super(origin, dest);
		this.isControlDep = isControlDep;
	}

	/**
	 * Returns if this Dependency/Edge is an Control Dependency
	 */
	public boolean isControlDep(){
		return isControlDep;
	}
	
	/**
	 * Returns if this Dependency/Edge is an Data Dependency
	 */
	public boolean isDataDep(){
		return !isControlDep;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((origin == null) ? 0 : origin.hashCode());
		result = prime * result + ((dest == null) ? 0 : dest.hashCode());
		result = prime * result + (isControlDep? 0 : 1);
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof DepEdge))
			return false;
		DepEdge other = (DepEdge) obj;
		if (origin == null) {
			if (other.origin != null)
				return false;
		} else if (!origin.equals(other.origin))
			return false;
		if (dest == null) {
			if (other.dest != null)
				return false;
		} else if (!dest.equals(other.dest))
			return false;
		return isControlDep == other.isControlDep;
	}

	@Override
	public DepEdge copy() {
		return new DepEdge(origin, dest, isControlDep);
	}

}
