package pt.moth.util.variables;

import soot.SootMethod;
import soot.jimple.Stmt;

/**
 * This Class represents a global variable that is present in a given if condition.
 * We need this kind of variables to find dependencies between views:
 * if(result(ViewA)...){
 * 		ViewB
 * }
 * 
 * ViewA  -->  ViewB
 * 		 Ifcond
 * 
 * @ifStmt, if Stmt where the variable was used
 * @global, Soot Object that has information about the global variable
 * @method, Method's name
 * 
 * @author Vasco Pessanha
 * 
 */
public class IfGlobalVar implements IfConditionVar{

	private GlobalVar global;
	private Stmt ifStmt;
	private SootMethod method;

	public IfGlobalVar(IfGlobalVar v) {
		this.global = v.global;
		this.ifStmt = v.ifStmt;
		this.method = v.method;
	}
	
	public IfGlobalVar(SootMethod method, GlobalVar global, Stmt ifStmt) {
		this.global = global;
		this.ifStmt = ifStmt;
		this.method = method;
	}
	public String toString(){
		return getName();
	}
	public String getName() {
		return global.getName()+"(if)";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((global == null) ? 0 : global.hashCode());
		result = prime * result
				+ ((ifStmt == null) ? 0 : ifStmt.toString().hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof IfGlobalVar))
			return false;
		IfGlobalVar other = (IfGlobalVar) obj;
		if (global == null) {
			if (other.global != null)
				return false;
		} else if (!global.equals(other.global))
			return false;
		if (ifStmt == null) {
			if (other.ifStmt != null)
				return false;
		} else if (!ifStmt.toString().equals(other.ifStmt.toString()))
			return false;
		return true;
	}

	@Override
	public IfGlobalVar copy() {
		return new IfGlobalVar(method,global.copy(),ifStmt);
	}

	@Override
	public Stmt getIfStmt() {
		return ifStmt;
	}
	
	public SootMethod getSootMethod(){
		return method;
	}
}
