package pt.moth.util.variables;

import soot.SootMethod;
import soot.jimple.internal.JimpleLocal;

/**
 * This Class represents a local variable. Despite the fact that the methods
 * equals and hashCode are implemented using only the name atribute, we keep the
 * Soot Objects for future using of their information
 * The name of a local variable consists in: (variable's method + "." + its name)
 * The atributes are:
 * 
 * @name, is the name of this variable ("method.name")
 * @local, Soot Object that has information about the local variable
 * @method, Method's name
 * 
 * @author Vasco Pessanha
 * 
 */
public class LocalVar implements Variable{

	private String name;
	private SootMethod method;
	private JimpleLocal local;

	public LocalVar(LocalVar v) {
		this.name = v.name;
		this.method = v.method;
		this.local = v.local;
	}
	
	public LocalVar(SootMethod method, JimpleLocal local) {
		this.method = method;
		this.local = local;
		this.name = method.getDeclaringClass()+"."+method.getName()+"."+local;
	}
	public String toString(){
		return name;
	}
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public JimpleLocal getJimpleLocal() {
		return local;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((local == null) ? 0 : local.hashCode());
		result = prime * result
				+ ((method == null) ? 0 : method.hashCode());
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof LocalVar))
			return false;
		LocalVar other = (LocalVar) obj;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		return true;
	}

	@Override
	public LocalVar copy() {
		return new LocalVar(method, local);
	}

	@Override
	public SootMethod getSootMethod() {
		return method;
	}
}
