package pt.moth.viewsAnalysis;

import java.util.List;

import soot.SootMethod;
import soot.jimple.Stmt;
import soot.toolkits.graph.DirectedGraph;
import soot.toolkits.graph.UnitGraph;
import soot.toolkits.scalar.FlowSet;
import soot.toolkits.scalar.ForwardFlowAnalysis;

public class IntraAnalysisNonAtomic<Unit> extends
		ForwardFlowAnalysis<Unit, Lattice> {

	protected InterAnalysis interAnalysis;
	protected SootMethod thisMethod;
	protected Lattice emptySet = new Lattice();

	public IntraAnalysisNonAtomic(DirectedGraph<Unit> graph,
			InterAnalysis interAnalysis) {
		super(graph);

		this.interAnalysis = interAnalysis;

		thisMethod = ((UnitGraph) graph).getBody().getMethod();
		
		doAnalysis();

	}

	@Override
	protected Lattice newInitialFlow() {
		return (Lattice)emptySet.clone();
	}

	@Override
	protected Lattice entryInitialFlow() {
		return (Lattice)emptySet.clone();
	}

	@Override
	protected void flowThrough(Lattice inValue, Object unit, Lattice outValue) {
		outValue.clear();
	
		if (((Stmt) unit).containsInvokeExpr()) {
			// handle method invocation: flow parameters into method call,
			// and flow return value from method call
			interAnalysis.handleMethodInvocation(thisMethod, new Lattice(),(Stmt) unit);
		}
		outValue.union(inValue);
	}

	@Override
	protected void merge(Lattice in1, Lattice in2,
			Lattice out) {
		FlowSet inSet1 = in1, inSet2 = in2, outSet = out;

		inSet1.union(inSet2, outSet);
	}

	@Override
	protected void copy(Lattice source, Lattice dest) {
		FlowSet sourceSet = source, destSet = dest;

		sourceSet.copy(destSet);
	}

	public Lattice getFlowAtEnd() {
		Lattice res = newInitialFlow();

		// merge the flowsets of all the method's exit-points
		List<Unit> tails = this.graph.getTails();
		for (Unit unit : tails) {
			Lattice sep1 = getFlowAfter(unit);
			if (sep1 == null) { // this method was not analyzed
				return res;
			}
			Lattice tmp = newInitialFlow();
			sep1.union(res, tmp);
			res = tmp;
		}
		return res;
	}
}
