package pt.moth.xmlAnnotations;

import java.util.HashSet;
import java.util.Set;

import soot.SootMethod;

/**
 * This Class handles the NATIVE methods. They should be analysed, annotated, ..?
 * @author Vasco Pessanha
 *
 */
public class NativesCollector {
	private static Set<SootMethod> native_methods = new HashSet<SootMethod>();
	private static Set<SootMethod> used_methods = new HashSet<SootMethod>();
	private static Set<SootMethod> excluded_methods = new HashSet<SootMethod>();
	private static final String[] native_to_exclude = {
		"java.util.Random","java.lang.Throwable","java.lang.Thread",
		"<java.lang.Object: java.lang.Class getClass()>",
		"java.lang.Class","java.lang.reflect.Array","sun.misc.",
		"java.security.AccessController","java.lang.Thread","java.lang.Shutdown"
	};
	
	
	public static void usedAnnotations(SootMethod nativeMethod){
		if(shouldAnnotate(nativeMethod)){
//			System.err.println("(NativesCollector) Used NATIVE method = "+nativeMethod.getSignature());
			used_methods.add(nativeMethod);
		}
	}
	
	public static void annotate(SootMethod nativeMethod, boolean hasObject){
		if(!shouldAnnotate(nativeMethod)){
			excluded_methods.add(nativeMethod);
		}
		else if(!wasAlreadyAnnotated(nativeMethod)){
//			System.err.println("(NativesCollector) Method "+nativeMethod.getSignature()+" is NATIVE!!!");
			NativeAnnotator.annotate(nativeMethod,hasObject);
			native_methods.add(nativeMethod);
		}
	}
	private static boolean wasAlreadyAnnotated(SootMethod nativeMethod){
		return native_methods.contains(nativeMethod);
	}
	private static boolean shouldAnnotate(SootMethod nativeMethod){
		for(int i = 0; i < native_to_exclude.length; i++){
			if(nativeMethod.getSignature().contains(native_to_exclude[i])){
				return false;
			}
		}
		return true;
	}
	
	public static Set<MethodName> getNativeMethods(){
		Set<MethodName> result = new HashSet<MethodName>();
		for(SootMethod m : native_methods){
			result.add(new MethodName(m));
		}
		return result;
	}
	public static Set<MethodName> getExcludedMethods(){
		Set<MethodName> result = new HashSet<MethodName>();
		for(SootMethod m : excluded_methods){
			result.add(new MethodName(m));
		}
		return result;
	}
	public static Set<MethodName> getUsedMethods(){
		Set<MethodName> result = new HashSet<MethodName>();
		for(SootMethod m : used_methods){
			result.add(new MethodName(m));
		}
		return result;
	}

	public static int getNumberOfNatives() {
		return native_methods.size();
	}
	public static int getNumberOfUsedNatives() {
		return used_methods.size();
	}
	public static int getNumberOfExcludedNatives() {
		return excluded_methods.size();
	}
}
